package com.liangjian.web.common.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.liangjian.web.common.entity.helper.PageWrapper;
import com.liangjian.web.common.entity.response.ResultObject;
import com.liangjian.web.common.service.crud.NameServiceMapper;
import com.liangjian.web.common.threadlocal.DynamicParamGetter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * Created by huangziqi on 2020/1/13
 */
@RestController
@RequestMapping("/crud/api")
@SuppressWarnings("unchecked")
public class RestfulCrudController {

    protected final NameServiceMapper nameServiceMapper;

    @Autowired
    public RestfulCrudController(NameServiceMapper nameServiceMapper) {
        this.nameServiceMapper = nameServiceMapper;
    }

    private void make404Response(HttpServletResponse response, String msg) {
        response.setHeader("Content-Type","application/json");
        response.setStatus(404);
        var res = Map.of("timestamp",System.currentTimeMillis(),
                "status","404",
                "error","Not Found",
                "message",msg);
        try {
            var js = new ObjectMapper().writeValueAsString(res);
            response.getWriter().write(js);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    @GetMapping("/{model}/{id}")
    public ResultObject<?> getSingleModel(HttpServletResponse response, @PathVariable("model") String model, @PathVariable("id") String id) {
        var opt = nameServiceMapper.getDynamicApiService(model);
        if(opt.isEmpty()) {
            make404Response(response, "can't not find suitable service for : " + model);
            return null;
        }

        return (ResultObject<?>) opt
                .get()
                .findById(id)
                .map(ResultObject::success)
                .orElse(ResultObject.error("model not found by id"));
    }

    @GetMapping("/{model}/page")
    public PageWrapper<?> getPagedModel(HttpServletResponse response, @PathVariable("model") String model) {
        var opt = nameServiceMapper.getDynamicApiService(model);
        if(opt.isEmpty()) {
            make404Response(response, "can't not find suitable service for : " + model);
            return null;
        }
        var params = DynamicParamGetter.DynamicParams.get();
        return opt.get().getPageByCondition(params);
    }

    @GetMapping("/{model}/list")
    public List<?> getListModel(HttpServletResponse response, @PathVariable("model") String model) {
        var opt = nameServiceMapper.getDynamicApiService(model);
        if(opt.isEmpty()) {
            make404Response(response, "can't not find suitable service for : " + model);
            return null;
        }
        var params = DynamicParamGetter.DynamicParams.get();
        return opt.get().getListByCondition(params);
    }

    @PostMapping("/{model}")
    public ResultObject<?> saveModel(HttpServletResponse response, @PathVariable("model") String model, @RequestBody String jsonStr) {
        var opt = nameServiceMapper.getDynamicApiService(model);
        if(opt.isEmpty()) {
            make404Response(response, "can't not find suitable service for : " + model);
            return null;
        }

        if (opt.get().addModel(jsonStr) ) {
            return ResultObject.success("add success",null);
        } else {
            return ResultObject.error("add error");
        }
    }

    @PutMapping("/{model}")
    public ResultObject<?> updateModel(HttpServletResponse response, @PathVariable("model") String model, @RequestBody String jsonStr) {
        var opt = nameServiceMapper.getDynamicApiService(model);
        if(opt.isEmpty()) {
            make404Response(response, "can't not find suitable service for : " + model);
            return null;
        }

        if (opt.get().updateModel(jsonStr) ) {
            return ResultObject.success("save success",null);
        } else {
            return ResultObject.error("save error");
        }
    }

    @DeleteMapping("/{model}/{id}")
    public ResultObject<?> deleteModelById(HttpServletResponse response, @PathVariable("model") String model, @PathVariable("id") String id) {
        var opt = nameServiceMapper.getDynamicApiService(model);
        if(opt.isEmpty()) {
            make404Response(response, "can't not find suitable service for : " + model);
            return null;
        }

        if(opt.get().deleteModelById(id)) {
            return ResultObject.success("delete success",null);
        } else {
            return ResultObject.error("delete error");
        }
    }
}
