package com.liangjian.web.common.interceptor;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.liangjian.web.common.dynamic.DynamicParams;
import com.liangjian.web.common.dynamic.DynamicParamsWorker;
import com.liangjian.web.common.entity.response.ResultObject;
import com.liangjian.web.common.threadlocal.DynamicParamGetter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Created by huangziqi on 2019/12/17
 *
 * 用于解析Get请求时携带的参数，动态查询的拦截器
 */
public class DynamicParamAnalyser implements HandlerInterceptor {
    Logger logger = LoggerFactory.getLogger(DynamicParamAnalyser.class);

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if(request.getMethod().equals("GET") && request.getRequestURI().startsWith("/crud/api")) {
            DynamicParams params = DynamicParamsWorker.analyze(request);
            if (params.getSize() > 100000) {
                logger.error("dynamic param's size is too large, size must be less than 100000. actual:{}",params.getSize());
                response.setHeader("Content-Type","application/json");
                String js = new ObjectMapper().writeValueAsString(ResultObject.error("dynamic param's size is too large, size must be less than 100000. actual:" + params.getSize()));
                response.getWriter().write(js);
                return false;
            }
            DynamicParamGetter.DynamicParams.set(params);
        }
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        DynamicParamGetter.DynamicParams.remove();
    }
}
