package com.liangjian.web.common.service;

import com.baomidou.mybatisplus.annotation.TableField;
import com.liangjian.web.common.entity.helper.EntityHelper;
import com.liangjian.web.common.util.LRUCache;

import javax.persistence.Column;
import java.lang.reflect.Field;
import java.util.*;

/**
 * Created by huangziqi on 2020/1/14
 */
public class EntityHelperBuilder {
    private LRUCache<Class, EntityHelper> cache = null;
    private final static EntityHelperBuilder instance = new EntityHelperBuilder();

    private EntityHelperBuilder() {
        cache = new LRUCache<>(20);
    }

    public static EntityHelperBuilder getInstance() {
        return instance;
    }

    /**
     * 根据class创建一个helper
     * @param clazz
     */
    private EntityHelper createHelper(Class clazz) {
        var list = Arrays.asList(clazz.getDeclaredFields());
        Map<String,Class> fieldType = new HashMap<>();

        for (Field field : list) {
            //直接把字段跟类型放进map中
            fieldType.put(field.getName(), field.getType());

            //是jpa中的字段注解
            if (field.isAnnotationPresent(Column.class)) {
                Column column = field.getAnnotation(Column.class);
                fieldType.put(column.name(), field.getType());
            }
            //mp的字段注解
            if(field.isAnnotationPresent(TableField.class)) {
                TableField tableField = field.getAnnotation(TableField.class);
                fieldType.put(tableField.value(), field.getType());
            }
        }
        EntityHelper helper = new EntityHelper() {
            Map<String,Class> map = fieldType;
            @Override
            public Optional<Class> getFieldClass(String field) {
                var value =  map.get(field);
                return Optional.ofNullable(value);
            }
        };
        return helper;
    }

    /**
     * 获取EntityHelper或者穿创建一个
     * @param clazz
     * @return
     */
    public EntityHelper getOrCreate(Class<?> clazz) {
        var helper = cache.get(clazz);
        if(helper == null) {
            synchronized (this) {
                helper = createHelper(clazz);
                cache.put(clazz, helper);
                return helper;
            }
        } else {
            return helper;
        }
    }
}
