package com.liangjian.web.common.service.crud;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import com.liangjian.web.common.entity.helper.EntityHelper;
import com.liangjian.web.common.service.EntityHelperBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Optional;

/**
 * Created by huangziqi on 2020/1/13
 */
public abstract class JpaDynamicApiService<T> extends JpaDynamicQueryService<T> {
    private static Logger log = LoggerFactory.getLogger(JpaDynamicApiService.class);

    public abstract JpaRepository<T,Serializable> getJpaRepository();

    public abstract Class<T> getModelClass();

    /**
     * 反序列化json
     * @param jsonStr
     * @return
     * @throws JsonProcessingException
     * @throws JsonMappingException
     */
    private T parseJson(String jsonStr) throws JsonProcessingException, JsonMappingException {
        Class<T> clazz = getModelClass();
        ObjectMapper mapper = new ObjectMapper();
        ObjectReader reader = mapper.readerFor(clazz);
        T model = reader.readValue(jsonStr);
        return model;
    }

    public void onAdd(T model) {
        log.warn("DynamicApiService for {} doesn't have a onAdd, it may save wired with jpa.", getMappedName());
    }

    public void onUpdate(T model) {
        log.warn("DynamicApiService for {} doesn't have a onUpdate, it may save wired with jpa.",getMappedName());
    }

    @Override
    public Optional<T> findById(Serializable id) {
        return getJpaRepository().findById(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean addModel(String json) {
        try {
            T model = parseJson(json);
            onAdd(model);
            getJpaRepository().save(model);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean updateModel(String json) {
        try {
            T model = parseJson(json);
            onUpdate(model);
            getJpaRepository().save(model);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean deleteModel(String json) {
        try {
            T model = parseJson(json);
            getJpaRepository().delete(model);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean deleteModelById(Serializable id) {
        try {
            getJpaRepository().deleteById(id);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    public EntityHelper getEntityHelper() {
        return EntityHelperBuilder.getInstance().getOrCreate(getModelClass());
    }
}
