package com.liangjian.web.common.service.crud;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import com.liangjian.web.common.entity.helper.EntityHelper;
import com.liangjian.web.common.service.EntityHelperBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Optional;

/**
 * Created by huangziqi on 2020/1/13
 */
public abstract class MpDynamicApiService<M extends BaseMapper<T>,T> extends MpDynamicQueryService<M,T> {
    private static Logger log = LoggerFactory.getLogger(MpDynamicApiService.class);

    public abstract Class<T> getModelClass();

    private T parseJson(String jsonStr) throws JsonProcessingException, JsonMappingException {
        Class<T> clazz = getModelClass();
        ObjectMapper mapper = new ObjectMapper();
        ObjectReader reader = mapper.readerFor(clazz);
        T model = reader.readValue(jsonStr);
        return model;
    }

    public void onAdd(T model) {
        log.warn("DynamicApiService for {} doesn't have a onAdd.", getMappedName());
    }

    public void onUpdate(T model) {
        log.warn("DynamicApiService for {} doesn't have a onUpdate.",getMappedName());
    }

    @Override
    public Optional<T> findById(Serializable id) {
        return Optional.ofNullable(getBaseMapper().selectById(id));
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean addModel(String json) {
        try {
            T model = parseJson(json);
            onAdd(model);
            int res = getBaseMapper().insert(model);
            if (res != 1) {
                log.error("insert error, result code : {}",res);
                throw new Exception("insert affected more than 1.");
            } else {
                return true;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean updateModel(String json) {
        try {
            T model = parseJson(json);
            onUpdate(model);
            int res = getBaseMapper().updateById(model);
            if (res != 1) {
                log.error("update error, result code: {}",res);
                throw new Exception("update affected more than 1.");
            } else {
                return true;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean deleteModel(String json) {
        log.error("unsupported deleteModel in mybatis.");
        var ex = new Exception("unsupported deleteModel in mybatis.");
        ex.printStackTrace();
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public boolean deleteModelById(Serializable id) {
        try {
            var res = getBaseMapper().deleteById(id);
            if(res != 1) {
                throw new Exception("delete affected more than 1.");
            } else {
                return true;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    public EntityHelper getEntityHelper() {
        return EntityHelperBuilder.getInstance().getOrCreate(getModelClass());
    }
}
