package com.liangjian.web.common.service.crud;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.liangjian.web.common.dynamic.DynamicFilter;
import com.liangjian.web.common.dynamic.DynamicParams;
import com.liangjian.web.common.dynamic.SortDirection;
import com.liangjian.web.common.entity.helper.EntityHelper;
import com.liangjian.web.common.entity.helper.PageWrapper;

import java.util.List;

/**
 * Created by huangziqi on 2020/1/13
 */
public abstract class MpDynamicQueryService<M extends BaseMapper<T>,T> extends ServiceImpl<M,T> implements DynamicApiService<T> {

//    protected abstract BaseMapper<T> getBaseMapper();

    protected QueryWrapper<T> toQueryWrapper(DynamicParams params) {
        EntityHelper helper = getEntityHelper();
        QueryWrapper<T> wrapper = new QueryWrapper<>();
        for (DynamicFilter filter : params.getFilters()) {
            switch (filter.getOperate()) {
                case Equal:
                    wrapper = wrapper.and(i -> i.eq(filter.getField(), filter.getValueBindType(helper)));
                    break;
                case NotEqual:
                    wrapper = wrapper.and(i -> i.ne(filter.getField(), filter.getValueBindType(helper)));
                    break;
                case Less:
                    wrapper = wrapper.and(i -> i.lt(filter.getField(), filter.getValueBindType(helper)));
                    break;
                case LessEqual:
                    wrapper = wrapper.and(i -> i.le(filter.getField(), filter.getValueBindType(helper)));
                    break;
                case Greater:
                    wrapper = wrapper.and(i -> i.gt(filter.getField(), filter.getValueBindType(helper)));
                    break;
                case GreaterEqual:
                    wrapper = wrapper.and(i -> i.ge(filter.getField(), filter.getValueBindType(helper)));
                    break;
                case Contains:
                    wrapper = wrapper.and(i -> i.like(filter.getField(), "%" + filter.getValue() + "%"));
                    break;
                case StartWith:
                    wrapper = wrapper.and(i -> i.like(filter.getField(), filter.getValue() + "%"));
                    break;
                case EndWith:
                    wrapper = wrapper.and(i -> i.like(filter.getField(), "%" + filter.getValue()));
                    break;
            }
        }
        return wrapper;
    }

    @Override
    public PageWrapper<T> getPageByCondition(DynamicParams params) {
        IPage<T> pager = new Page<>(params.getPage(),params.getSize());
        QueryWrapper<T> wrapper = toQueryWrapper(params);
        if(params.getSortField() != null && !params.getSortField().isEmpty()) {
            if(params.getSortDirection() == SortDirection.ASC) {
                wrapper.orderByAsc(params.getSortField());
            } else {
                wrapper.orderByDesc(params.getSortField());
            }
        }
        IPage<T> result = getBaseMapper().selectPage(pager,wrapper);
        return PageWrapper.wrap(result);
    }

    @Override
    public List<T> getListByCondition(DynamicParams params) {
        QueryWrapper<T> wrapper = toQueryWrapper(params);
        if(params.getSortField() != null && !params.getSortField().isEmpty()) {
            if(params.getSortDirection() == SortDirection.ASC) {
                wrapper.orderByAsc(params.getSortField());
            } else {
                wrapper.orderByDesc(params.getSortField());
            }
        }
        return getBaseMapper().selectList(wrapper);
    }
}
